package networking;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import model.Direction;

public class GameBoardServer extends GameBoardThreading {
    private final DataInputStream[] inputs;
    private final DataOutputStream[] outputs;
    private final int refreshRate;

    /**
     * Creates a new GameBoard with the given scale, and 
     * each client is represented with a Data Input/Output Stream pair
     * @param scale The scale of the board
     * @param is Client end points to read from
     * @param os Client end points to write to
     */
    public GameBoardServer(int refreshRate, InputStream[] is, OutputStream[] os) {
        super("Server");
        this.refreshRate = refreshRate;
        inputs = new DataInputStream[is.length];
        outputs = new DataOutputStream[os.length];
        for (int i = 0; i < is.length; i++){
            inputs[i] = new DataInputStream(is[i]);
        }
        for (int i = 0; i < os.length; i++){
            outputs[i] = new DataOutputStream(os[i]);
        }
    }
    
    @Override
    protected void logic() {
        System.out.println("Server: started");
        byte boardBytes[] = toByteArray();
        boolean changes = false;
        while (!Thread.interrupted()){
            for (int i = 0; i < inputs.length; i++){
                DataInputStream dis = inputs[i];
                try {
                    if (dis != null && dis.available() > 0){
                        Direction d = Direction.values()[dis.readByte()];
                        changePlayerDirection(i, d);
                        System.out.println("Server: Client-" + i + " turns to " + d);
                        changes = true;
                    }
                } catch (IOException ex) {
                    System.out.println("Server: Connection to Client-" + i + " is lost (read)");
                    inputs[i] = null;
                    outputs[i] = null;
                }
            }
            if (changes){
                movePlayers();
                boardBytes = toByteArray();
            }
            
            for (int i = 0; i < outputs.length; i++){
                try {
                    if (outputs[i] != null) outputs[i].write(boardBytes);
                } catch (IOException ex) {
                    System.out.println("Server: Connection to Client-" + i + " is lost (write)");
                    inputs[i] = null;
                    outputs[i] = null;
                }
            }                
            try {
                Thread.sleep(refreshRate);
            } catch (InterruptedException ex) {
                System.out.println("Server: interrupted");
                break;
            }
        }
    }
}
